%This function compute the upper and lower uncertainty error over the
%computation of the heat transfer coefficient using the Jackson and Hall
%correlation in convection. The error depends on the bulk fluid temeprature
%T_F ( and its errors  dT_F), the surface temperature measured by the IR
%camera T_surf, the absolute pressure P_sCO2, the mass flux G_chan, the
%Reynolds number Re_b, the hydraulic diameter and the exponent of the corrective term n_val. 
function [dh_JandH_up,dh_JandH_down] = uncertainty_h_JandH(T_F,dT_F_up,dT_F_down,T_surf,P_sCO2,G_chan,Re_b,D_h,h_JandH,n_val_JH,fluid)
%error of the IR camera
if T_surf -273.15 > 100
    error_T_surf = 0.02*T_surf;
else
    error_T_surf = 2;
end
error_P =  20000; %absolute uncertainty of the pressure transducer
error_G = 15.6732 + 0.001*G_chan;

mu = CoolProp.PropsSI('V', 'T', T_F, 'P', P_sCO2, fluid);             %Viscosity
rho_wall = CoolProp.PropsSI('D', 'T', T_surf, 'P', P_sCO2, fluid);%Density at the wall temperature
rho_bulk = CoolProp.PropsSI('D', 'T', T_F, 'P', P_sCO2, fluid);   %Density at the bulk temperature
Pr_bulk = CoolProp.PropsSI('Prandtl', 'T', T_F, 'P', P_sCO2, fluid);%Prandtl number at the bulk temperature
cp_bulk = CoolProp.PropsSI('C', 'T', T_F, 'P', P_sCO2, fluid);%Specific heat at constant pressure at the bulk temperature
h_bulk =  CoolProp.PropsSI('H', 'T', T_F, 'P', P_sCO2, fluid);%Density at the wall temperature
h_wall =  CoolProp.PropsSI('H', 'T', T_surf, 'P', P_sCO2, fluid);%Density at the wall temperature
k_bulk = CoolProp.PropsSI('conductivity', 'T', T_F, 'P', P_sCO2, fluid);%Thermal onductivity at the bulk temperature

%% dh_JandH_up
%T_F impact
mu_T_F_up = CoolProp.PropsSI('V', 'T', T_F + dT_F_up, 'P', P_sCO2, fluid);
Re_b_T_F_up = (G_chan*D_h)/mu_T_F_up;
rho_bulk_T_F_up = CoolProp.PropsSI('D', 'T', T_F + dT_F_up , 'P', P_sCO2, fluid);
Pr_bulk_T_F_up = CoolProp.PropsSI('Prandtl', 'T', T_F + dT_F_up , 'P', P_sCO2, fluid);
cp_bulk_T_F_up = CoolProp.PropsSI('C', 'T', T_F + dT_F_up, 'P', P_sCO2, fluid);
h_bulk_T_F_up =  CoolProp.PropsSI('H', 'T', T_F + dT_F_up, 'P', P_sCO2, fluid);
k_bulk_T_F_up = CoolProp.PropsSI('conductivity', 'T', T_F + dT_F_up, 'P', P_sCO2, fluid);

A = h_wall-h_bulk_T_F_up;
B = T_surf-(T_F + dT_F_up);
cp_bar = (A)/(B);
Nu_JandH_T_F_up = 0.0183*Re_b_T_F_up^(0.82)*Pr_bulk_T_F_up^(0.5)*(rho_wall/rho_bulk_T_F_up)^(0.3)*(cp_bar/cp_bulk_T_F_up)^(n_val_JH);
d_T_F_up = (Nu_JandH_T_F_up*k_bulk_T_F_up)/D_h - h_JandH;

%T_surf impact
rho_wall_T_surf_up = CoolProp.PropsSI('D', 'T', T_surf + error_T_surf, 'P', P_sCO2, fluid);
h_wall_T_surf_up =  CoolProp.PropsSI('H', 'T', T_surf + error_T_surf, 'P', P_sCO2, fluid);
A = h_wall_T_surf_up-h_bulk;
B = T_surf + error_T_surf-T_F;
cp_bar = (A)/(B);
Nu_JandH_T_surf_up = 0.0183*Re_b^(0.82)*Pr_bulk^(0.5)*(rho_wall_T_surf_up/rho_bulk)^(0.3)*(cp_bar/cp_bulk)^(n_val_JH);
d_T_surf_up = (Nu_JandH_T_surf_up*k_bulk)/D_h - h_JandH;

%Pressure impact
mu_P_up = CoolProp.PropsSI('V', 'T', T_F, 'P', P_sCO2 + error_P, fluid);  
Re_b_P_up = (G_chan*D_h)/mu_P_up;
rho_wall_P_up = CoolProp.PropsSI('D', 'T', T_surf, 'P', P_sCO2 + error_P, fluid);
rho_bulk_P_up = CoolProp.PropsSI('D', 'T', T_F, 'P', P_sCO2 + error_P, fluid);   
Pr_bulk_P_up = CoolProp.PropsSI('Prandtl', 'T', T_F, 'P', P_sCO2 + error_P, fluid);
cp_bulk_P_up = CoolProp.PropsSI('C', 'T', T_F, 'P', P_sCO2 + error_P, fluid);
h_bulk_P_up =  CoolProp.PropsSI('H', 'T', T_F, 'P', P_sCO2 + error_P, fluid);
h_wall_P_up =  CoolProp.PropsSI('H', 'T', T_surf, 'P', P_sCO2 + error_P, fluid);
k_bulk_P_up = CoolProp.PropsSI('conductivity', 'T', T_F, 'P', P_sCO2 + error_P, fluid);

A = h_wall_P_up-h_bulk_P_up;
B = T_surf-T_F;
cp_bar = (A)/(B);
Nu_JandH_P_up = 0.0183*Re_b_P_up^(0.82)*Pr_bulk_P_up^(0.5)*(rho_wall_P_up/rho_bulk_P_up)^(0.3)*(cp_bar/cp_bulk_P_up)^(n_val_JH);
d_P_up = (Nu_JandH_P_up*k_bulk_P_up)/D_h - h_JandH;

%G impact
Re_b_G_up = ((G_chan+error_G)*D_h)/mu;
A = h_wall-h_bulk;
B = T_surf-T_F;
cp_bar = (A)/(B);
Nu_JandH_G_up = 0.0183*Re_b_G_up^(0.82)*Pr_bulk^(0.5)*(rho_wall/rho_bulk)^(0.3)*(cp_bar/cp_bulk)^(n_val_JH);
d_G_up = (Nu_JandH_G_up*k_bulk)/D_h - h_JandH;

dh_JandH_up = sqrt(d_T_F_up^2 + d_T_surf_up^2 + d_P_up^2 + d_G_up^2);

%% dh_JandH_down
%T_F impact
mu_T_F_down = CoolProp.PropsSI('V', 'T', T_F - dT_F_down, 'P', P_sCO2, fluid);
Re_b_T_F_down = (G_chan*D_h)/mu_T_F_down;
rho_bulk_T_F_down = CoolProp.PropsSI('D', 'T', T_F - dT_F_down , 'P', P_sCO2, fluid);
Pr_bulk_T_F_down = CoolProp.PropsSI('Prandtl', 'T', T_F - dT_F_down , 'P', P_sCO2, fluid);
cp_bulk_T_F_down = CoolProp.PropsSI('C', 'T', T_F - dT_F_down, 'P', P_sCO2, fluid);
h_bulk_T_F_down =  CoolProp.PropsSI('H', 'T', T_F - dT_F_down, 'P', P_sCO2, fluid);
k_bulk_T_F_down = CoolProp.PropsSI('conductivity', 'T', T_F - dT_F_down, 'P', P_sCO2, fluid);

A = h_wall-h_bulk_T_F_down;
B = T_surf-(T_F - dT_F_down);
cp_bar = (A)/(B);
Nu_JandH_T_F_down = 0.0183*Re_b_T_F_down^(0.82)*Pr_bulk_T_F_down^(0.5)*(rho_wall/rho_bulk_T_F_down)^(0.3)*(cp_bar/cp_bulk_T_F_down)^(n_val_JH);
d_T_F_down = (Nu_JandH_T_F_down*k_bulk_T_F_down)/D_h - h_JandH;

%T_surf impact
rho_wall_T_surf_down = CoolProp.PropsSI('D', 'T', T_surf - error_T_surf, 'P', P_sCO2, fluid);
h_wall_T_surf_down =  CoolProp.PropsSI('H', 'T', T_surf - error_T_surf, 'P', P_sCO2, fluid);
A = h_wall_T_surf_down-h_bulk;
B = T_surf - error_T_surf-T_F;
cp_bar = (A)/(B);
Nu_JandH_T_surf_down = 0.0183*Re_b^(0.82)*Pr_bulk^(0.5)*(rho_wall_T_surf_down/rho_bulk)^(0.3)*(cp_bar/cp_bulk)^(n_val_JH);
d_T_surf_down = (Nu_JandH_T_surf_down*k_bulk)/D_h - h_JandH;

%Pressure impact
mu_P_down = CoolProp.PropsSI('V', 'T', T_F, 'P', P_sCO2 - error_P, fluid);  
Re_b_P_down = (G_chan*D_h)/mu_P_down;
rho_wall_P_down = CoolProp.PropsSI('D', 'T', T_surf, 'P', P_sCO2 - error_P, fluid);
rho_bulk_P_down = CoolProp.PropsSI('D', 'T', T_F, 'P', P_sCO2 - error_P, fluid);   
Pr_bulk_P_down = CoolProp.PropsSI('Prandtl', 'T', T_F, 'P', P_sCO2 - error_P, fluid);
cp_bulk_P_down = CoolProp.PropsSI('C', 'T', T_F, 'P', P_sCO2 - error_P, fluid);
h_bulk_P_down =  CoolProp.PropsSI('H', 'T', T_F, 'P', P_sCO2 - error_P, fluid);
h_wall_P_down =  CoolProp.PropsSI('H', 'T', T_surf, 'P', P_sCO2 - error_P, fluid);
k_bulk_P_down = CoolProp.PropsSI('conductivity', 'T', T_F, 'P', P_sCO2 - error_P, fluid);

A = h_wall_P_down-h_bulk_P_down;
B = T_surf-T_F;
cp_bar = (A)/(B);
Nu_JandH_P_down = 0.0183*Re_b_P_down^(0.82)*Pr_bulk_P_down^(0.5)*(rho_wall_P_down/rho_bulk_P_down)^(0.3)*(cp_bar/cp_bulk_P_down)^(n_val_JH);
d_P_down = (Nu_JandH_P_down*k_bulk_P_down)/D_h - h_JandH;

%G impact
Re_b_G_down = ((G_chan-error_G)*D_h)/mu;
A = h_wall-h_bulk;
B = T_surf-T_F;
cp_bar = (A)/(B);
Nu_JandH_G_down = 0.0183*Re_b_G_down^(0.82)*Pr_bulk^(0.5)*(rho_wall/rho_bulk)^(0.3)*(cp_bar/cp_bulk)^(n_val_JH);
d_G_down = (Nu_JandH_G_down*k_bulk)/D_h - h_JandH;

dh_JandH_down = sqrt(d_T_F_down^2 + d_T_surf_down^2 + d_P_down^2 + d_G_down^2);
