%Function to compute the buoyancy correlation by Petukhov and Polyakov and
%the Jackson criteria for the acceleration.
% Inlet: PLate temperature T_m, fluid temperature T_F, absolute pressure
% P_sCO2, the fluid, the heat geenrated E_gen, the geometrical value dx
% (width) and dy (length), the hydraulic diameter D_h, the mass flux
% G_chan, the perimeter per and the heated perimeter per_heated.
function [Bo_star,K_v] = buoyancy_acc(T_m,T_F,P_sCO2,fluid,E_gen,dx,dy,D_h,G_chan,per,per_heated)
g = 9.81;                                                         %Acceleration due to gravity
q_flux = E_gen/(dx*dy);                                            %Heat flux (W/m^2)
T_film = (T_m + T_F)/2;
h_bulk = CoolProp.PropsSI('H', 'T', T_F+273.15, 'P', P_sCO2, fluid); %Specific enthalpy of the bulk fluid
h_wall = CoolProp.PropsSI('H', 'T', T_m+273.15, 'P', P_sCO2, fluid); %Specific enthalpy at the wall temperature
mu_bulk = CoolProp.PropsSI('viscosity', 'T', T_F+273.15, 'P', P_sCO2, fluid); %Dynamic viscosity at the bulk temperature
k_bulk = CoolProp.PropsSI('conductivity', 'T', T_F+273.15, 'P', P_sCO2, fluid);%Thermal onductivity at the bulk temperature
rho_bulk = CoolProp.PropsSI('D', 'T', T_F+273.15, 'P', P_sCO2, fluid);%Density at the bulk temperature
rho_wall = CoolProp.PropsSI('D', 'T', T_m+273.15, 'P', P_sCO2, fluid);%Density at the wall temperature
rho_film = CoolProp.PropsSI('D', 'T', T_film+273.15, 'P', P_sCO2, fluid);%Density at the film temperature
nu_bulk = mu_bulk/rho_bulk;                                                  %Kinematic viscosity
Re_b = (G_chan*D_h)/mu_bulk;                                           %Reynolds number
Beta_bulk = CoolProp.PropsSI('ISOBARIC_EXPANSION_COEFFICIENT', 'T', T_F+273.15, 'P', P_sCO2, fluid);%Volume expansion coefficient at the bulk temperature
cp_bulk = CoolProp.PropsSI('C', 'T', T_F+273.15, 'P', P_sCO2, fluid);%Specific heat at constant pressure at the bulk temperature


%Buoyancy correlation by Petukhov and Polyakov
Pr_bar = (h_wall-h_bulk)/(T_m-T_F) * (mu_bulk/k_bulk);                           %Integrated Prandtl number
Beta_bar= 1/(rho_film)* ((rho_bulk-rho_wall)/(T_m-T_F));
Gr_q= (g*Beta_bar*q_flux*D_h^4)/(nu_bulk^2*k_bulk);
Gr_th=3e-5*Re_b^(2.75)*Pr_bar^(0.5)*(1+2.4*Re_b^(-1/8)*(Pr_bar^(2/3)-1));
Bo_star = Gr_q/Gr_th;


%Acceleration correlation by McEligoth and Jackson for asymmetric heating
q_plus = Beta_bulk*q_flux/(G_chan*cp_bulk);
K_v = 4*(per_heated/per)*q_plus/Re_b;
end