%Title : Convective analysis transient
%Author: Bertrand Schyns
%Model of analysis of the IR camera data in additioon to the LabView data of
%the corresponding test in transient regime.


clc
clear all

Frame = 'G490-570_5s.xlsx'%Name of the LabView data file

%As the number of frames is different from one test to the other, a
%keyboard input is used 
prompt = 'How many frames is there to analyse? \n';
z = input(prompt);
prompt = 'Are the data points for a transient heat flux? (Yes:1 , No:0) \n'; %The camera was moved between the mass flux test and the heat flux test so the pixel selection is different between the two.
y = input(prompt);
%%
%Loading the Inconel properties that will be needed to compute the
%cnduction and the heat production. Those values were calculated using EES
%and will be interpolated later on. The units used are kg/K/Pa/J like
%Coolprop. The temperature goes from 1 to 200C with a 1C step.
Inconel_props = xlsread('Inconel_prop.xlsx', 'A2:C202');
Inconel_props = Inconel_props';
Inconel_T = Inconel_props(1,:);
Inconel_k = Inconel_props(2,:);
Inconel_rho_e = Inconel_props(3,:);

%%
%Geometrical properties and problem specification
Th_B=254*10^(-6);                        %Thickness of the bottom Inconel 718sheet (micron to m)
L_B= 50*10^(-3);                         %Length of the bottom Inconel 718 sheet(mm to m) seen by the camera in the gap /!\ask Saad for actual dimension
W_ch= 1.92*10^(-3);                         %Width of the flow channel (mm to m)
Th_SW=1.5*10^(-3);                       %Width of the wall seperating the channels (mm to m)
Th_SE=1.5*10^(-3);                       %Width of the side edge used for bonding the Inconel 718 to Macor (mm to m)
N_ch=3;                                  %Number of channel
R_a= 0.78*10^(-6);                          %Absolute roughness of the channel walls (micron to m)
W_header = N_ch*W_ch+Th_SW*(N_ch-1)+2*Th_SE;%Width of the header at the inlet and outlet
L_header = 7*10^(-3);                    %Length of the header region at the inlet and outlet
W_total = (2*Th_SE)+(N_ch-1)*Th_SW+N_ch*W_ch;	%Total width of the test section
A_cross_total = W_total*Th_B;            %Total cross sectional area for current flow

%Flow channel hydraulic diameter
H_ch= 600*10^(-6);                       %Height of the flow channels (micron to m)
A_cross_flow = H_ch*W_ch;                %Flow cross sectional area
per= 2*(H_ch+W_ch);                      %Perimeter
D_h= (4*A_cross_flow)/per;               %Hydraulic diamter

%Flow conditions, the pressure and the mass flow rate is assumed constant
%in this case as the heat flux variation are kept relatively small
fluid = 'CO2';
%All data from Labview. The usefull columns are here :
%3.Inlet temperature [C] 4.Outlet temperature [C] 7.Inlet pressure [Psi]
%10.Mass flow rate [kg/h]
flow_data = xlsread(Frame,'data');
P_sCO2_psi=mean(flow_data(:,18));                        %Absolute pressure in the test section in psi
P_sCO2 =convpres(P_sCO2_psi,'psi','Pa'); %Absolute pressure in the test section in Pa
P_sCO2_bar= P_sCO2/10^5;                 %Absolute pressure in the test section in bar
G_chan = xlsread(Frame,'MassFlux'); %Mass flux in the test section [kg/m^2-s]
m_dot_chan= G_chan*A_cross_flow;         %Mass flow rate in a single channel [kg/s]
m_dot_total=m_dot_chan*N_ch;             %Total mass flow rate through the test section
G_total=G_chan;

 I_curr=xlsread(Frame,'current');                          %Current through the test section [Amp]

%Defining the pseudo-critical temperature
T_PC= -122.6+6.124*P_sCO2_bar-0.1657*P_sCO2_bar^2+0.01773*P_sCO2_bar^(2.5)-0.0005608*P_sCO2_bar^3;  %Pseudo-critical temperature - Liao and Zhao (2002) [C]

%%

%run analyse_IR to get the temperature of the Inconnel sheet at the
%different times
T_frame = zeros(8,243,z);
for i = 1:z
    [T_channel] = analyse_IR(y,i);
    T_frame(:,:,i) = fliplr(T_channel(3:10,6:248));%
    T_max(i) = max(max(T_frame(:,:,i)));
    
    
end
size_chan = size(T_frame);

%Segmenting the model in the third dimension
N_nodes = size_chan(2);                            %Number of nodes in the bottom portion of the test section
N_nodes_cross =size_chan(1);                      %Number of node accros the channel"
L_begin = 0;                             %Left edge of the test section [m]
dy= L_B/(N_nodes);                      %Length of a nodes along the y axis (flow direction)
dx = W_ch/(N_nodes_cross);                 %Width of a node along the x axis (perpendicular to the flow direction)

%Assigning the position to the nodes
L_ax = zeros(1,length(N_nodes));
for i= 1:N_nodes
    L_ax(i)= L_begin+ dy *(i-1);
end
A_cross = dx*Th_B;
V_elem = A_cross*dy;

%Radiation
epsilon=0.95;                            %emissivity
sigma = 5.670374419*10^(-8);             %Stefan Boltzmann constant [W/m^2.K^4]
T_inf = 23;                              %Room temperature [C]

for m = 1:z
    for i = 1:size_chan(1)
        for j = 1:size_chan(2)
            T_m(i,j) = T_frame(i,j,m);
        end
    end
    
    T_F_in(m) = flow_data(m+4,3);       %Inlet temperature [C]. The +4 stand for the 2s delay between the start of LabView and the start of the camera and so the different frames
    h_F_in(m) = CoolProp.PropsSI('H', 'T', T_F_in(m)+273.15, 'P', P_sCO2, fluid);  %[J/kg]
    [dh_F_in_up(m),dh_F_in_down(m)] = uncertainty_h_F(T_F_in(m),h_F_in(m),P_sCO2,fluid);
    
    T_F_out(m) = flow_data(m+4,4);
    h_F_out(m) = CoolProp.PropsSI('H', 'T', T_F_out(m)+273.15, 'P', P_sCO2, fluid);
    [dh_F_out_up(m),dh_F_out_down(m)] = uncertainty_h_F(T_F_out(m),h_F_out(m),P_sCO2,fluid);
    
    
    Delta_P_H2O(m) = flow_data(m+4,17);
    Delta_kPa(m) =Delta_P_H2O(m)*0.2491; %Conversion from inch H2O to kPa
    
    
    %Current
    I_density(m)= I_curr(m)/A_cross_total;
    I_elem(m) = I_density(m) * A_cross;           %Current flowing through a single element
    I_sw(m) = I_density(m) * Th_SW * Th_B ;
   
    
    
    %%
    if m == 1 %Initialization of matrix
        %Boundary conditions
        %The fluid temperature is considered uniform accros the width of the channel
        T_F = zeros(z,N_nodes);         %Fluid temperature
        h_F = zeros(z,N_nodes);         %Fluid enthalpy
        dh_F_up = zeros(z,N_nodes);     %Upper error on the enthalpy of the fluid due to the different measurement errors
        dh_F_down = zeros(z,N_nodes);   %Lower error on the enthalpy of the fluid due to the different measurement errors
        dT_F_up = zeros(z,N_nodes);     %Upper error on the temperature of the fluid due to the different measurement errors
        dT_F_down = zeros(z,N_nodes);   %Lower error on the temperature of the fluid due to the different measurement errors
        
        
        %
        E_gen_sw = zeros(z,N_nodes);      %Heat flux equal at both side, the only difference along the channel is the electric resistivity
        E_gen = zeros(N_nodes_cross,N_nodes,z);     %Heat generated in the Inconel
        k = zeros(N_nodes_cross,N_nodes,z);         %Conductivity of the Inconel at every node
        rho_e = zeros(N_nodes_cross,N_nodes,z);   %Electrical conductivity of the Inconel at every node
        dE_gen_sw_up = zeros(z,N_nodes); %Upper error on the energy generated due to the different measurement errors
        dE_gen_sw_down = zeros(z,N_nodes); %Lower error on the energy generated due to the different measurement errors
        dE_gen_up = zeros(N_nodes_cross,N_nodes,z); %Upper error on the energy generated due to the different measurement errors
        dE_gen_down = zeros(N_nodes_cross,N_nodes,z); %Lower error on the energy generated due to the different measurement errors
        E_gen_header_in = zeros(1,z); %Heat produced in the inlet header region
        dE_gen_header_in_up = zeros(1,z); %Upper error on the energy generated in the header region 
        dE_gen_header_in_down = zeros(1,z); %Lower error on the energy generated in the header region
        E_gen_header_out = zeros(1,z); %Heat produced in the outlet header region
        dE_gen_header_out_up = zeros(1,z); %Upper error on the energy generated in the header region 
        dE_gen_header_out_down = zeros(1,z); %Lower error on the energy generated in the header region
        
        
        %
        q_rad = zeros(N_nodes_cross,N_nodes,z);     %Heat transfer by radiation
        q_conv = zeros(N_nodes_cross,N_nodes,z);    %Heat transfer by convection
        h_conv = zeros(N_nodes_cross,N_nodes,z);  %Convective heat transfer coefficient
        dq_rad_up = zeros(N_nodes_cross,N_nodes,z); %Upper error on heat exchanged by radiation due to the different measurement errors
        dq_rad_down = zeros(N_nodes_cross,N_nodes,z);%Lower error on heat exchanged by radiation due to the different measurement errors
        dq_conv_up = zeros(N_nodes_cross,N_nodes,z);%Upper error on heat exchanged by convection due to the different measurement errors
        dq_conv_down = zeros(N_nodes_cross,N_nodes,z); %Lower error on heat exchanged by convection due to the different measurement errors
        dh_conv_up = zeros(N_nodes_cross,N_nodes,z); %Upper error on the convective heat transfer coefficient due to the different measurement errors
        dh_conv_down = zeros(N_nodes_cross,N_nodes,z);%Lower error on the convective heat transfer coefficient due to the different measurement errors
        
        h_F0 = zeros(1,z); %Enthalpy of the sCO2 at the inlet of the channels
        dh_F0_up = zeros(1,z); %Upper error on the enthalpy of the sCO2
        dh_F0_down = zeros(1,z);%Lower error on the enthalpy of the sCO2
        T_F0 = zeros(1,z); %Temperature of the sCO2 at the inlet of the channels
        
        mu_b = zeros(z,length(N_nodes));    %Dynamic viscosity at the bulk temperature
        Re_b = zeros(z,length(N_nodes));    %Reynolds number at the bulk fluid temperature
        
        Bo_star = zeros(N_nodes_cross,N_nodes,z);   %Buoyancy parameter 
        K_v = zeros(N_nodes_cross,N_nodes,z);       %Acceleration parameter
        
        h_F_out_calc = zeros(1,z); %Enthalpy of the sCO2 at the outlet
        dh_F_out_up = zeros(1,z); %Upper error on the enthalpy of the sCO2 at the outlet
        dh_F_out_down = zeros(1,z);%Lower error on the enthalpy of the sCO2 at the outlet
        T_F_out_calc  = zeros(1,z); %Temperature of the sCO2 at the outlet
        dT_F_out_calc_up = zeros(1,z);%Upper error on the temperature of the sCO2 at the outlet
        dT_F_out_calc_down = zeros(1,z);%Lower error on the temperature of the sCO2 at the outlet
    end
    
    for i = 1:N_nodes
        position_sw = floor(T_m(1,i)+10);
        %+10 is there to represent the fact that the temperature in the side
        %wall will be generally higher as it is not directly cooled down
        diff_sw = T_m(1,i) +10 - position_sw;
        rho_e_sw = Inconel_rho_e(position_sw) + diff_sw*(Inconel_rho_e(position_sw+1) - Inconel_rho_e(position_sw));
        E_gen_sw(m,i) = I_sw(m)^2 * dy * rho_e_sw / (Th_SW * Th_B);
        [dE_gen_sw_up(i,m), dE_gen_sw_down(i,m)] = uncertainty_E_gen(I_sw(m),T_m(1,i)+10,rho_e_sw,dy,Th_SW * Th_B,Inconel_rho_e,E_gen_sw(m,i));
        for j = 1:N_nodes_cross
            position = floor(T_m(j,i));
            diff = T_m(j,i) - position;
            k(j,i,m) = Inconel_k(position) + diff*(Inconel_k(position+1) - Inconel_k(position));
            rho_e(j,i,m) = Inconel_rho_e(position) + diff*(Inconel_rho_e(position+1) - Inconel_rho_e(position));
            E_gen(j,i,m) = I_elem(m)^2 * dy * rho_e(j,i,m) / A_cross;
            [dE_gen_up(j,i,m), dE_gen_down(j,i,m)] = uncertainty_E_gen(I_elem(m),T_m(j,i),rho_e(j,i,m),dy,A_cross,Inconel_rho_e,E_gen(j,i,m));
        end
    end
    E_gen_header_in(m) = I_curr(m)^2 * L_header * mean(rho_e(:,1,m))/(Th_B*W_header);
    [dE_gen_header_in_up(m), dE_gen_header_in_down(m)] = uncertainty_E_gen(I_curr(m),mean(T_m(:,1)),mean(rho_e(:,1,m)),L_header,Th_B*W_header,Inconel_rho_e,E_gen_header_in(m));
    E_gen_header_out(m) = I_curr(m)^2 * L_header * mean(rho_e(:,end,m))/(Th_B*W_header);
    [dE_gen_header_out_up(m), dE_gen_header_out_down(m)] = uncertainty_E_gen(I_curr(m),mean(T_m(:,end)),mean(rho_e(:,end,m)),L_header,Th_B*W_header,Inconel_rho_e,E_gen_header_out(m));
    
    
    %%
    %Equations
        
    for i = 1: N_nodes
        %Heat exchange is computed for all the channel points
        for j = 1: N_nodes_cross
            q_rad(j,i,m)= epsilon * sigma * dx * dy * ((T_m(j,i)+273.15)^4 - (T_inf+273.15)^4);
            q_conv(j,i,m) = E_gen(j,i,m) - q_rad(j,i,m)  + 4/3*E_gen_sw(m,i)/N_nodes_cross;
            [dq_rad_up(j,i,m), dq_rad_down(j,i,m)] = uncertainty_q_rad(T_m(j,i), epsilon, sigma, dx, dy, T_inf, q_rad(j,i,m));
            dq_conv_up(j,i,m) = sqrt(dE_gen_up(j,i,m)^2 +  dq_rad_up(j,i,m)^2 + (4/3*dE_gen_sw_up(m,i)/N_nodes_cross)^2) ;
            dq_conv_down(j,i,m) = sqrt(dE_gen_down(j,i,m)^2 +  dq_rad_down(j,i,m)^2 + (4/3*dE_gen_sw_down(m,i)/N_nodes_cross)^2);
        end
        
        %Energy conservation
        %First the enthalpy is computed and then the temperature is computed using
        %Coolprop. The '-' sign is used as the convective heat transfer is
        %computed with regard to the solid and not the fluid.
        if i == 1
            h_F0(m) = (m_dot_chan(m) * N_ch *h_F_in(m) + E_gen_header_in(m))/(m_dot_chan(m) * N_ch);
            [dh_F0_up(m), dh_F0_down(m)] = uncertainty_h_bo(m_dot_chan(m) * N_ch,h_F0(m),h_F_in(m),dh_F_in_up(m),dh_F_in_down(m),E_gen_header_in(m),dE_gen_header_in_up(m), dE_gen_header_in_down(m));
            T_F0(m) =CoolProp.PropsSI('T', 'H', h_F0(m), 'P', P_sCO2, fluid)-273.15;
            h_F(m,i) = (m_dot_chan(m) * h_F0(m) + sum(q_conv(:,i,m)) )/m_dot_chan(m);
            [dh_F_up(m,i), dh_F_down(m,i)] = uncertainty_h(m_dot_chan(m),h_F(m,i),h_F0(m),dh_F0_up(m),dh_F0_down(m),E_gen(:,i,m),dE_gen_up(:,i,m), dE_gen_down(:,i,m),E_gen_sw(m,i),dE_gen_sw_up(m,i),dE_gen_sw_down(m,i),q_rad(:,i,m),dq_rad_up(:,i,m), dq_rad_down(:,i,m));
        else
            h_F(m,i) = (m_dot_chan(m) * h_F(m,i-1) + sum(q_conv(:,i,m)))/m_dot_chan(m);
            [dh_F_up(m,i), dh_F_down(m,i)] = uncertainty_h(m_dot_chan(m),h_F(m,i),h_F(m,i-1),dh_F_up(m,i-1),dh_F_down(m,i-1),E_gen(:,i,m),dE_gen_up(:,i,m), dE_gen_down(:,i,m),E_gen_sw(m,i),dE_gen_sw_up(m,i),dE_gen_sw_down(m,i),q_rad(:,i,m),dq_rad_up(:,i,m), dq_rad_down(:,i,m));
        end
        T_F(m,i) =CoolProp.PropsSI('T', 'H', h_F(m,i), 'P', P_sCO2, fluid)-273.15;
        [dT_F_up(m,i),dT_F_down(m,i)] = uncertainty_T_F(T_F(m,i),h_F(m,i),dh_F_up(m,i), dh_F_down(m,i),P_sCO2,fluid);
        mu_b(m,i) = CoolProp.PropsSI('viscosity', 'T', T_F(m,i)+273.15, 'P', P_sCO2, fluid); %Dynamic viscosity at the bulk temperature
        Re_b(m,i) = (G_chan(m)*D_h)/mu_b(m,i);                                           %Reynolds number
        
        h_conv(:,i,m) = q_conv(:,i,m)./(dx*dy*(T_m(:,i)-T_F(m,i)));
        for j = 1: N_nodes_cross
            [dh_conv_up(j,i,m),dh_conv_down(j,i,m)] = uncertainty_h_conv(h_conv(j,i,m),q_conv(j,i,m),dq_conv_up(j,i,m),dq_conv_down(j,i,m),T_m(j,i),T_F(m,i),dT_F_up(m,i),dT_F_down(m,i),dx,dy);
            [Bo_star(j,i,m),K_v(j,i,m)] = buoyancy_acc(T_m(j,i),T_F(m,i),P_sCO2,fluid,E_gen(j,i,m)+4/3*E_gen_sw(m,i)/N_nodes_cross,dx,dy,D_h,G_chan(m),per,W_ch);
        end
        
    end
    
    h_F_out_calc(m) = (m_dot_chan(m) * N_ch * h_F(m,end) + E_gen_header_out(m))/(m_dot_chan(m) * N_ch);
    [dh_F_out_up(m), dh_F_out_down(m)] = uncertainty_h_bo(m_dot_chan(m) * N_ch,h_F_out_calc(m),h_F(m,end),dh_F_up(m,end),dh_F_down(m,end),E_gen_header_out(m),dE_gen_header_out_up(m), dE_gen_header_out_down(m));
    T_F_out_calc(m) =CoolProp.PropsSI('T', 'H', h_F_out_calc(m), 'P', P_sCO2, fluid)-273.15;
    [dT_F_out_calc_up(m),dT_F_out_calc_down(m)] = uncertainty_T_F(T_F_out_calc(m),h_F_out_calc(m),dh_F_out_up(m), dh_F_out_down(m),P_sCO2,fluid);
    
    E_gen_tot(m) = E_gen_header_in(m) + E_gen_header_out(m) + sum(E_gen_sw(m,:))*4 + sum(sum(E_gen(:,:,m)))*3;
    Q_dot(m) =  E_gen_tot(m)/(W_header*((L_header*2)+L_B));
end



%% Figures
T_R = (T_F+273.15)/(T_PC+273.15);
x = (0:dy:(length(mean(h_conv))-1)*dy)*1000;


for i = 1 : z
    time(i)=i/2;
    h_time(i) = mean(mean(h_conv(:,:,i)));
    dh_time_up(i) =  mean(mean(dh_conv_up(:,:,i)));
    dh_time_down(i) =  mean(mean(dh_conv_down(:,:,i)));
    Tm_time(i) = mean(mean(T_frame(:,:,i)));
    Bo_time(i) = mean(mean(Bo_star(:,:,i)));
end

figure
plot(time,h_time,'o')
grid
xlabel('Time (s)')
ylabel('Mean heat transfer coefficient (W/m^2K)')

figure
plot(time,Tm_time,'o')
grid
xlabel('Time (s)')
ylabel('Mean plate temperature (C)')


figure
plot(time,Bo_time,'o')
grid
xlabel('Time (s)')
ylabel('Gr_q/Gr_{th}')


%% Plot from the LabView data
TF_in = flow_data(:,3);
TF_out = flow_data(:,4);
t = [0.5:0.5:length(TF_in)/2];

figure
plot(t,TF_in,'o')
hold on
plot(t,TF_out,'o')
grid
xlabel ('Time (s)')
ylabel ('Fluid temperature (C)')
legend('T_{inlet} measured','T_{outlet} measured')


%% Figures for a specific frame m
% figure
% a = mean(h_conv(m,:));
% a1 = mean(dh_conv_down(m,:));
% a2 = mean(dh_conv_up(m,:));
% errorbar(x(1:2:end),a(1:2:end),a2(1:2:end),a2(1:2:end),'o') %'LineStyle','none'
% 
% figure
% errorbar(x(1:2:end),T_F(m,1:2:end),dT_F_down(m,1:2:end),dT_F_up(m,1:2:end),'o')
% hold on
% errorbar(((L_header)+L_B)*1000,T_F_out_calc(m),dT_F_out_calc_down(m),dT_F_out_calc_up(m),'o')
% fplot(T_F_in(m))
% fplot(T_F_out(m))
% fplot(T_PC,':')
% legend('Computed temperature','T_{out} (computed)','T_{in}','T_{out}','T_{PC}')
% axis([0 58 (T_F_in(m)-1) (T_F_out(m)+4)])
% 
% 
% figure
% error_T_m = zeros(1,length(mean(T_m)));
% error_T_m(:) = 2;
% b = mean(T_frame(:,:,m));
% errorbar(x(1:2:end),b(1:2:end),error_T_m(1:2:end),error_T_m(1:2:end),'o')
% 
% figure
% c = mean(Bo_star(m));
% d = mean(K_v(m));
% yyaxis left
% plot(x(1:2:end),c(1:2:end),'o')
% yyaxis right
% plot(x(1:2:end),d(1:2:end),'o')
% legend('Buoyancy criterion (threshold value: 1)','Acceleration crtierion (threshold value: 9.5*10^{-7})')
